@@ -14,7 +14,8 @@ import jax.tree_util as jtu
 import numpy as np
 from jax.core import Tracer
 
-from pytreeclass._src.tree_util import _tree_fields, is_treeclass, tree_copy
+import pytreeclass as pytc
+from pytreeclass._src.tree_util import tree_copy
 
 PyTree = Any
 
@@ -88,14 +89,14 @@ def _field_boolean_map(
         # however if we get the tree_leaves of the tree, `b` will not be visible to the condition.
 
         for field_item, node_item in (
-            [f, getattr(tree, k)]
-            for k, f in _tree_fields(tree).items()
+            [f, getattr(tree, f.name)]
+            for f in pytc.fields(tree)
             if not f.metadata.get("static", False)
         ):
 
             yield from _true_leaves(node_item) if cond(field_item, node_item) else (
                 _traverse(node_item)
-                if is_treeclass(node_item)
+                if pytc.is_treeclass(node_item)
                 else _false_leaves(node_item)
             )
 
