@@ -4,20 +4,18 @@ import ctypes
 from dataclasses import Field
 from typing import Any
 
+import pytreeclass as pytc
 import pytreeclass._src as src
-from pytreeclass._src.tree_util import (
-    _tree_fields,
-    is_frozen_field,
-    is_nondiff_field,
-    is_treeclass,
-    is_treeclass_frozen,
-    is_treeclass_nondiff,
-)
 from pytreeclass.tree_viz.node_pprint import _format_node_diagram
 from pytreeclass.tree_viz.tree_export import _generate_mermaid_link
+from pytreeclass.tree_viz.tree_summary import (
+    _format_count,
+    _format_size,
+    _reduce_count_and_size,
+)
 
 
-def _marker(field_item: Field, node_item: Any, default: str = "---") -> str:
+def _marker(field_item: Field, node_item: Any, default: str = "--") -> str:
     """return the suitable marker given the field and node item
 
     Args:
@@ -29,49 +27,69 @@ def _marker(field_item: Field, node_item: Any, default: str = "---") -> str:
         str: marker character.
     """
     # for now, we only have two markers '*' for non-diff and '#' for frozen
-    if is_nondiff_field(field_item) or is_treeclass_nondiff(node_item):
+    if pytc.is_nondiff_field(field_item) or pytc.is_treeclass_nondiff(node_item):
         return "--x"
-    elif is_frozen_field(field_item) or is_treeclass_frozen(node_item):
+    elif pytc.is_frozen_field(field_item) or pytc.is_treeclass_frozen(node_item):
         return "-..-"
-    else:
-        return default
+    return default
 
 
 PyTree = Any
 
 
+def _bold_text(text: str) -> str:
+    # bold a text in ansci code
+    return "<b>" + text + "</b>"
+
+
 def _tree_mermaid(tree: PyTree):
     def node_id(input):
         """hash a node by its location in a tree"""
         return ctypes.c_size_t(hash(input)).value
 
     def recurse(tree, depth, prev_id):
-        if not is_treeclass(tree):
+        if not pytc.is_treeclass(tree):
             return
 
         nonlocal FMT
 
-        for i, field_item in enumerate(_tree_fields(tree).values()):
+        cur_id = node_id((depth, -1, prev_id))
 
-            if not field_item.repr:
+        for i, field_item in enumerate(pytc.fields(tree)):
+
+            if not field_item.repr or (pytc.is_nondiff_field(field_item)):
                 continue
 
             node_item = getattr(tree, field_item.name)
+            count, size = _reduce_count_and_size(node_item)
+
+            if count.real == 0:
+                count = size = ""
+            else:
+                count = (
+                    _format_count(count.real)
+                    + " param"
+                    + ("s" if count.real > 1 else "")
+                )
+                size = _format_size(size.real)
             cur_id = node_id((depth, i, prev_id))
 
             if isinstance(node_item, src.tree_base._treeBase):
                 mark = _marker(field_item, node_item, default="--->")
-                FMT += f"\n\tid{prev_id} {mark} id{cur_id}({field_item.name}\\n{node_item.__class__.__name__})"
+                FMT += f"\n\tid{prev_id} {mark} "
+                FMT += f'|"{(count)}<br>{(size)}"| '
+                FMT += f'id{cur_id}("{_bold_text(field_item.name) }<br>{node_item.__class__.__name__}")'
                 recurse(tree=node_item, depth=depth + 1, prev_id=cur_id)
-
             else:
                 mark = _marker(field_item, node_item, default="----")
-                FMT += f'\n\tid{prev_id} {mark}  id{cur_id}["{field_item.name}\\n{_format_node_diagram(node_item)}"]'
+                FMT += f"\n\tid{prev_id} {mark} "
+                FMT += f'|"{(count)}<br>{(size)}"| '
+                FMT += f'id{cur_id}["{_bold_text(field_item.name)}<br>{_format_node_diagram(node_item)}"]'
 
         prev_id = cur_id
 
     cur_id = node_id((0, 0, -1, 0))
-    FMT = f"flowchart LR\n\tid{cur_id}[{tree.__class__.__name__}]"
+    FMT = f"flowchart LR\n\tid{cur_id}({_bold_text(tree.__class__.__name__)})"
     recurse(tree=tree, depth=1, prev_id=cur_id)
     return FMT.expandtabs(4)
 
